/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/
package com.vektorsoft.demux.core.mva;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * Base class for specific implementations of
 * <code>ViewManager</code>. Provides some common methods implementation that
 * can be reused by subclasses.
 */
public abstract class AbstractViewManager implements DMXViewManager {

    /**
     * Contains all views registered with this manager. Map key is v iew name,
     * and value is the view itself.
     */
    private Map<String, DMXView> registeredViews;
    
    /** Views pending registration until their parent view is registered. */
    private Map<String, Set<String>> deferredViews;
    
    /**
     * Contains map of data which are contained by the view. Map key is view ID,
     * and value is a set of data IDs.
     */
    private Map<String, Set<String>> viewDataContents;
    
    /**
     * Maps data IDs to a set of view IDs that refer to this data. Map key is data ID,
     *  and value is a set of view IDs.
     */
    private Map<String, Set<String>> dataToViewMapping;
    
    /** Adapter instance associated with this view manager. */
    protected DMXAdapter adapter;

    /**
     * Create new instance.
     */
    protected AbstractViewManager(){
        registeredViews = new ConcurrentSkipListMap<String, DMXView>();
        deferredViews = new ConcurrentSkipListMap<String, Set<String>>();
        viewDataContents = new ConcurrentSkipListMap<String, Set<String>>();
        dataToViewMapping = new ConcurrentSkipListMap<String, Set<String>>();
    }
    
    /**
     * Called to complete registration of deferred views. Subclasses should override this method
     * to complete view registration, if required.
     * 
     * @param parentId registered parent view
     * @param childrenIds ids of dependent views
     */
    protected abstract void processDeferredViews(String parentId, Set<String> childrenIds);
    
    /**
     * Registers specified view for management by this view manager.
     * 
     * @param view view to register
     */
    @Override
    public  void registerView(DMXView view) {
        registeredViews.put(view.getViewId(), view);
        // register view data with the manager
        Set<String> viewDataIds = view.getViewDataIds();
        viewDataContents.put(view.getViewId(), viewDataIds);
        for(String entry : view.getViewDataIds()){
            
            // add this view to data-to-view mapping
            if(dataToViewMapping.containsKey(entry)){
                dataToViewMapping.get(entry).add(view.getViewId());
            } else {
                Set<String> viewIds = new HashSet<String>();
                viewIds.add(view.getViewId());
                dataToViewMapping.put(entry, viewIds);
            }
        }
        // check if parent view is registered and save this view to deffered if needed
        String parentId = view.getParentViewId();
        if(parentId != null && !parentId.isEmpty()){
            boolean registered = registeredViews.containsKey(parentId);
            if(!registered){
                if(deferredViews.containsKey(parentId)){
                    deferredViews.get(parentId).add(view.getViewId());
                } else {
                    Set<String> ids = new HashSet<String>();
                    ids.add(view.getViewId());
                    deferredViews.put(parentId, ids);
                }
            }
        }
        // check if this view is needed for deffered views to complete registration
        String viewId = view.getViewId();
        if(deferredViews.containsKey(viewId)){
            processDeferredViews(viewId, deferredViews.get(viewId));
            deferredViews.remove(viewId);
        }
    }

    /**
     * @see DMXViewManager#getView(java.lang.String) 
     * 
     * @param name view name
     * @return  view
     */
    @Override
    public DMXView getView(String name) {
        return registeredViews.get(name);
    }

    /**
     * @see DMXViewManager#getViewDataIds(java.lang.String) 
     * @param viewId view ID
     * @return data IDs registered for specified view
     */
    @Override
    public Set<String> getViewDataIds(String viewId) {
        return viewDataContents.get(viewId);
    }

    /**
     * {@inheritDoc }.
     */
    @Override
    public Set<String> getViewIdsForData(String dataId) {
        return dataToViewMapping.get(dataId);
    }
    
}
