/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.core.mva;

import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * <p> Represents the Adapter in Model-View-Adapter pattern. This class is
 * responsible for communication and synchronization of View and Model. It is
 * also used to register individual controllers and invoke them when
 * appropriate. </p>
 */
public class DMXAdapter implements Observer {

    /**
     * <code>DMXViewManager</code> instance that is used to manage views.
     */
    protected DMXViewManager viewManager;
    /**
     * Map of all registered controllers.
     */
    protected Map<String, DMXController> controllerMap;
    
    /** Model to be used with this adapter. */
    protected DMXModel dataModel;

    /**
     * Create new instance.
     */
    public DMXAdapter() {
        controllerMap = new ConcurrentSkipListMap<String, DMXController>();
        this.dataModel = new DMXModel();
        dataModel.registerObserver(this);
    }

    /**
     * Sets the current view manager.
     * @param vm <code>DMXViewManager</code> to set
     */
    public void setViewManager(DMXViewManager vm) {
        this.viewManager = vm;
    }

    /**
     * Sets current dataModel.
     * 
     * @param model dataModel to set
     */
    public void setModel(DMXModel model) {
        this.dataModel = model;
        dataModel.registerObserver(this);
    }

    /**
     * Registers specified controller to be used by the Adapter. Each controller
     * needs to be registered in order to be used properly. This method will do the following:
     * <ol>
     *  <li>Add controller to a list of registered controllers</li>
     *  <li>Register all controller data with the dataModel</li>
     * </ol>
     * 
     * @param controller controller to register
     */
    public void registerController(DMXController controller) {
        //check parameters validity
        if(controller == null){
            throw new IllegalArgumentException("Controller cannot be null.");
        }
        if(controller.getMapping() == null || "".equals(controller.getMapping())){
            throw new IllegalArgumentException("Controller mapping cannot be null or empty string.");
        }
        
        controllerMap.put(controller.getMapping(), controller);
        for(Map.Entry<String, Object> entry : getControllerData(controller).entrySet()){
            dataModel.registerData(entry.getKey(), entry.getValue());
        }
    }
    
    /**
     * Registers this view with this application. This method will delegate view registration 
     * to {@link DMXViewManager}, and register view data with dataModel.
     * 
     * @param view view to register
     */
    public void registerView(DMXView view){
        viewManager.registerView(view);
    }

    /**
     * Extracts data from controller that should be register with the dataModel.
     *
     * @param controller controller
     * @return map of controller data
     */
    protected Map<String, Object> getControllerData(DMXController controller) {
       return controller.getControllerDataMap();
    }

    /**
     * <p>
     * Invoked when any of the dataModel data changes. Adapter will notify view manager, which
     *  will in turn update views that depend on changed data.
     * </p>
     * <p>
     *  Argument must be an instance of {@code java.util.Map}, or the {@code IllegalArgumentException} 
     * will be thrown.
     * </p>
     * 
     * @param o source of the change
     * @param arg optional argument
     */
    @Override
    public void update(Observable o, Object arg) {
        // assume that argument is map
        if(arg instanceof Map){
            viewManager.updateViewData((Map<String, Object>)arg);
        } else {
            throw new IllegalArgumentException("Argumetn must be an instance of java.util.Map");
        }
    }
    
     /**
     * Invokes controller with specified ID. This method is called whenever controller needs to be executed. It will
     * call controllers <code>execute()</code> method with arguments provided.
     * 
     * @param controllerId controller ID
     * @param args execution arguments
     */
    public void invokeController(String controllerId, Object... args) {
        try {
            DMXController controller = controllerMap.get(controllerId);
            if(controller == null){
                throw new NoSuchControllerException("Controller not found: " + controllerId);
            }
            Set<String> names = controller.getControllerDataNames();
            Map<String, Object> dataMap = dataModel.getDataValues(names.toArray(new String[names.size()]));
            
            controller.updateFromModel(dataMap);
            controller.execute(args);
            dataModel.setDataValues(controller.updateModel());
            
        } catch (ExecutionException ex) {
            throw new RuntimeException(ex);
        } catch (NoSuchControllerException nex){
            throw new RuntimeException(nex);
        }
    }
}
